<?php

	/**
	 * TwistPHP - An open source PHP MVC framework built from the ground up.
	 * Copyright (C) 2016  Shadow Technologies Ltd.
	 *
	 * This program is free software: you can redistribute it and/or modify
	 * it under the terms of the GNU General Public License as published by
	 * the Free Software Foundation, either version 3 of the License, or
	 * (at your option) any later version.
	 *
	 * This program is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 *
	 * You should have received a copy of the GNU General Public License
	 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
	 *
	 * @author     Shadow Technologies Ltd. <contact@shadow-technologies.co.uk>
	 * @license    https://www.gnu.org/licenses/gpl.html GPL License
	 * @link       https://twistphp.com
	 */

	namespace Twist\Core\Models\String;

	class SyntaxHighlight{

		/**
		 * Get the contents of a file and highlight the code within the file, set a range above and below a focus point (Line No)
		 * @param string $dirFile
		 * @param string $strOutputType Choose from plain, em, table, dl (Plain puts line numbers in html comments)
		 * @param null|int $intFocusLineNo Adding a focus line no. will activate $intFocusRange
		 * @param int $intFocusRange Amount of lines to display above and below the focus line
		 * @return string
		 */
		public static function file($dirFile,$strOutputType = 'plain',$intFocusLineNo = null,$intFocusRange = 3){
			return (file_exists($dirFile)) ? self::processCode(file_get_contents($dirFile),$strOutputType,$intFocusLineNo,$intFocusRange) : '';
		}

		/**
		 * Hightlight some code that has been passed in, set a range above and below a focus point (Line No)
		 * @param string $strCode
		 * @param string $strOutputType Choose from plain, em, table, dl (Plain puts line numbers in html comments)
		 * @param null|int $intFocusLineNo Adding a focus line no. will activate $intFocusRange
		 * @param int $intFocusRange Amount of lines to display above and below the focus line
		 * @return string
		 */
		public static function code($strCode,$strOutputType = 'plain',$intFocusLineNo = null,$intFocusRange = 3){
			return self::processCode($strCode,$strOutputType,$intFocusLineNo,$intFocusRange);
		}

		/**
		 * Process the code that has been passed in, highlight and output
		 * @param string $strCode
		 * @param string $strOutputType Choose from plain, em, table, dl (Plain puts line numbers in html comments)
		 * @param null|int $intFocusLineNo Adding a focus line no. will activate $intFocusRange
		 * @param int $intFocusRange Amount of lines to display above and below the focus line
		 * @return string
		 */
		protected static function processCode($strCode,$strOutputType,$intFocusLineNo,$intFocusRange){

			$strOut = '';
			$intLineNo = 0;

			//Get all the lines of code as individual lines
			$arrCodeLines = self::explodeLines(highlight_string($strCode, true));

			$intStartLine = (!is_null($intFocusLineNo)) ? $intFocusLineNo-$intFocusRange : 0;
			$intEndLine = (!is_null($intFocusLineNo)) ? $intFocusLineNo+$intFocusRange : count($arrCodeLines);

			switch($strOutputType){

				case 'em':
					$strCodeContainer = "<code>%s</code>";
					$strLineContainer = "<em%s>%s</em> | <span%s>%s</span>\n";
					break;

				case 'em-no-container':
					$strCodeContainer = "%s";
					$strLineContainer = "<em%s>%s</em> | <span%s>%s</span>\n";
					break;

				case 'table':
					$strCodeContainer = "<table>\n%s</table>";
					$strLineContainer = "<tr><th%s>%s</th><td%s>%s</td></tr>";
					break;

				case 'dl':
					$strCodeContainer = "<dl>\n%s</dl>";
					$strLineContainer = "<dt%s>%s</dt><dd%s>%s</dd>";
					break;

				case 'plain-no-container':
					$strCodeContainer = "%s";
					$strLineContainer = "<!-- %s, Line: %s --><span%s>%s</span>\n";
					break;

				case 'plain':
				default:
					$strCodeContainer = "<code>%s</code>";
					$strLineContainer = "<!-- %s, Line: %s --><span%s>%s</span>\n";
					break;
			}

			$intLineNoPadLength = strlen($intEndLine);
			foreach($arrCodeLines as $strEachLine){
				$intLineNo++;

				//Only output lines that are within the correct range
				if($intLineNo >= $intStartLine && $intLineNo <= $intEndLine){

					$strFocusLine = ($intLineNo == $intFocusLineNo) ? ' class="highlight"' : '';
					$strOut .= sprintf($strLineContainer,$strFocusLine,str_pad($intLineNo,$intLineNoPadLength,' ',STR_PAD_RIGHT),$strFocusLine,self::convertStyles($strEachLine));
				}
			}

			return sprintf($strCodeContainer,$strOut);
		}

		/**
		 * Explode the highlighted code that has been returned, fix any unclosed/unopened tags
		 * @param string $strCode
		 * @return array Each line of highlighted code as an array
		 */
		protected static function explodeLines($strCode){

			//Pre-pair highlighted code
			$strCode = str_replace(array("<code>","</code>"),"",$strCode);
			$strCode = str_replace(array("<br>","<br />","<br >"),"\n",$strCode);

			$arrCodeLines = explode("\n",$strCode);

			//Loose the first and last 2 lines as they are the base color, we will re-add that to the main container
			array_shift($arrCodeLines);
			array_pop($arrCodeLines);

			$strLastOpenTag = $strNextLastOpenTag = $strCodeLine = '';
			$intPosition = 0;

			foreach($arrCodeLines as $intPosition => $strCodeLine){

				preg_match_all('#(<[\/]?span( style="[^\"]*")?>)#i', $strCodeLine, $arrSpanTags);

				$intCloses = $intOpens = 0;
				$strFirstTag = null;

				foreach($arrSpanTags[2] as $intSearchPosition => $strTagType){
					if($strTagType == ''){
						$intCloses++;
						$strFirstTag = (is_null($strFirstTag)) ? 'close' : $strFirstTag;
					}else{
						$intOpens++;
						$strFirstTag = (is_null($strFirstTag)) ? 'open' : $strFirstTag;
						$strNextLastOpenTag = $arrSpanTags[1][$intSearchPosition];
					}
				}

				if($intOpens != $intCloses){
					$strCodeLine = ($strFirstTag == 'open') ? $strCodeLine.'</span>' : $strLastOpenTag.$strCodeLine;
				}elseif($intOpens == 0 && $intCloses == 0 || $strFirstTag == 'close'){
					$strCodeLine = $strLastOpenTag.$strCodeLine.'</span>';
				}

				$strLastOpenTag = $strNextLastOpenTag;

				//Remove any empty span tags (Clean up essentially)
				$arrCodeLines[$intPosition] = preg_replace('#<span style="[^\"]*"></span>#i',"",$strCodeLine);
			}

			//Check the last line of code entered, if it is now empty (where not before) then remove it.
			if($arrCodeLines[$intPosition] == '' && $strCodeLine != ''){
				array_pop($arrCodeLines);
			}

			return $arrCodeLines;
		}

		/**
		 * Apply new styles to the code rather than the default ones
		 * @param string $strFormattedCode
		 * @return mixed
		 */
		protected static function convertStyles($strFormattedCode){ //TODO: Write some code to detect the default styles and replace them with custom ones if required
			return $strFormattedCode;
		}
	}
